/*****************************************************
The functions below translate the diagram of a knot or  link into
the system of equations, following the alternative method for computing the hyperbolic structure of the link from its diagram. The code was written and fully tested for daigrams with 2, 3 and 4-sided regions, but can be modified for a k-sided region. Contributor: A. Tsvietkova, University of Tennessee
*****************************************************/


#include "link_diagram.h"
#include <iostream>
using namespace std;


/*********************************************************
This function assigns standard (i.e regular) initial values
to edge_numbers and crossing_numbers variables. For the
2-sided regions these values will be changed (to 0's and
1's) by the next function f. 
*********************************************************/
void Link::initial_numbers(complex<double> *x) 
   {  int i,s,k;
      double g=3;
      for ( i=1; i<=crossing_number; ++i )
	      { 
			  if (crossingList[i]->sign == 1) 
			     x[i-1] = complex<double>(0, 0.5); //positive crossings (orientation) 
			  else
				 x[i-1] = complex<double>(0, -0.5); //negative ones
	     
			  crossingList[i]->index_in_x = i-1; //index of the crossing number in the final solution vector
	  
	      }
        
	  for ( i=(crossing_number+1); i<=crossing_number*3; ++i )    //num_edges=num_crossings*2
	      { x[i-1] = complex<double>(-0.5,0.5);//first n edges are from the black regions, later put them into edgeList[i]->u_black
	        edgeList[i-crossing_number]->black_index_in_x = i-1;
	      } 
	   
	  for ( i=(3*crossing_number+1); i<=crossing_number*5; ++i )    //num_edges=num_crossings*2
	      { x[i-1] = complex<double>(0.5,0.5);// next n edges are from the white regions, later put these values into edgeList[i]->u_white
	        edgeList[i-3*crossing_number]->white_index_in_x = i-1;
	      } 

  //Change initial values for 2-sided regions from the regular ones to 0, 1 and -1
      for ( s=1; s<=(crossing_number+2); ++s )    //there are (crossing_number+2) regions in the link/knot
         {
          Quadrant *current_quadrant = baseQuadrantList[s]; //Each region is in one-to-one correspondence with baseQuadrant; current_cudrant is used for choosing the region; baseQuadrantList[i] has the crossing crossingList[i]
          Quadrant *region_quadrant = baseQuadrantList[s];  //region_quadrant is used for taking all crossings in the chosen region
          if (current_quadrant->region_num_sides == 2)     //if the region is 2-sided, change the initial values
		      {for (k=1; k<=current_quadrant->region_num_sides; ++k)
				      { if (region_quadrant->crossing->nw->region_index == region_quadrant->region_index)
				            { if ( region_quadrant->color == black )
				                 { region_quadrant->crossing->west->edge->u_black = complex<double>(0,0);
				                   x[region_quadrant->crossing->west->edge->black_index_in_x]=complex<double>(0,0);
                                   region_quadrant->crossing->west->edge->u_white = complex<double>(1,0);
								   x[region_quadrant->crossing->west->edge->white_index_in_x]=complex<double>(1,0);
				                 }
					          else
						         { region_quadrant->crossing->west->edge->u_white = complex<double>(0,0);
                                   x[region_quadrant->crossing->west->edge->white_index_in_x]=complex<double>(0,0);
                                   region_quadrant->crossing->west->edge->u_black = complex<double>(-1,0);
								   x[region_quadrant->crossing->west->edge->black_index_in_x]=complex<double>(-1,0);	 
				                 }
						     }
					    else if (region_quadrant->crossing->ne->region_index == region_quadrant->region_index)
				            { if ( region_quadrant->color == black )
				                 { region_quadrant->crossing->north->edge->u_black = complex<double>(0,0);
				                   x[region_quadrant->crossing->north->edge->black_index_in_x]=complex<double>(0,0);
                                   region_quadrant->crossing->north->edge->u_white = complex<double>(1,0);
								   x[region_quadrant->crossing->north->edge->white_index_in_x]=complex<double>(1,0);	 
				                 }
					          else
						         { region_quadrant->crossing->north->edge->u_white = complex<double>(0,0);
                                   x[region_quadrant->crossing->north->edge->white_index_in_x]=complex<double>(0,0);
                                   region_quadrant->crossing->north->edge->u_black = complex<double>(-1,0);
								   x[region_quadrant->crossing->north->edge->black_index_in_x]=complex<double>(-1,0);	
				                 }
						     }
                        else if (region_quadrant->crossing->se->region_index == region_quadrant->region_index)
				            { if ( region_quadrant->color == black )
				                 { region_quadrant->crossing->east->edge->u_black = complex<double>(0,0);
				                   x[region_quadrant->crossing->east->edge->black_index_in_x]=complex<double>(0,0);
                                   region_quadrant->crossing->east->edge->u_white = complex<double>(1,0);
								   x[region_quadrant->crossing->east->edge->white_index_in_x]=complex<double>(1,0);
						         }
					          else
						         { region_quadrant->crossing->east->edge->u_white = complex<double>(0,0);
                                   x[region_quadrant->crossing->east->edge->white_index_in_x]=complex<double>(0,0);
                                   region_quadrant->crossing->east->edge->u_black = complex<double>(-1,0);
								   x[region_quadrant->crossing->east->edge->black_index_in_x]=complex<double>(-1,0);
						         }
						     }
                        else if (region_quadrant->crossing->sw->region_index == region_quadrant->region_index)
				            { if ( region_quadrant->color == black )
				                 { region_quadrant->crossing->south->edge->u_black = complex<double>(0,0);
				                   x[region_quadrant->crossing->south->edge->black_index_in_x]=complex<double>(0,0);
                                   region_quadrant->crossing->south->edge->u_white = complex<double>(1,0);
								   x[region_quadrant->crossing->south->edge->white_index_in_x]=complex<double>(1,0);
						         }
					          else
						         { region_quadrant->crossing->south->edge->u_white = complex<double>(0,0);
                                   x[region_quadrant->crossing->south->edge->white_index_in_x]=complex<double>(0,0);
                                   region_quadrant->crossing->south->edge->u_black = complex<double>(-1,0);
								   x[region_quadrant->crossing->south->edge->black_index_in_x]=complex<double>(-1,0);
						         }
						     }
				  
				        region_quadrant = region_quadrant->next;
				      }  //end of for			 			 
		       } //end of if about 2-sided region
        } //end of for 



   }





/*******************************************************
This function makes equations for every region depending
on how many sides the region has. It also calculates 
derivatives and makes jacobian matrix for the system. 
The function gives an output as a vector of residuals fv
and the rectangular jacobian matrix jac. The input is a vector 
of initial values x and its length n.
*********************************************************/
void Link::f(complex<double> *x, complex<double> *fv, int n, complex<double> **jac)   //n is a number of variables
  {   
      int s, k, p; 
	  int h = 0, t = 0; 
	  complex<double> A[2][2], B[2][2], C[2][2];
	  complex<double> *help_massive_u, *help_massive_w, *help_massive_ksi;  //help_massive store u_i, w_i's for a chosen region
	  int *help_massive_u_index, *help_massive_w_index;  //these store the indexes of u_i's and w_i's in solution vector x
	  complex<double> *help_massive_eps;   //epsilon_i	


      //Below I just put initial values from array x to the link  
	  for ( s=1; s<=crossing_number; ++s )   //crossingList and edgeList start with index 1, as well as x and fv
	      { crossingList[s]->w = x[s-1];     //first variables in massive x are crossing numbers, massive x starts with index 0, edgeList with 1 
	      }
      for ( s=(crossing_number+1); s<=3*(crossing_number); ++s )    //num_edges=num_crossings*2
	      { edgeList[s - crossing_number]->u_black = x[s-1];     //after crossing numbers in massive x there are edge numbers from the black regions 
	      }
      for ( s=(3*crossing_number+1); s<=5*(crossing_number); ++s )    //num_edges=num_crossings*2
	      { edgeList[s - 3*crossing_number]->u_white = x[s-1];     //then there are edge numbers from the white regions 
	      }
      // end

      //Below is the first set of equations: u_i's on two sides of the same edge differ by 1 
      for ( s=1; s<=2*(crossing_number); ++s) 
	      { fv[h] = edgeList[s]->u_white - edgeList[s]->u_black - complex<double>(1,0);
	        
	        //Compute partial derivatives, store them in jacobian matrix  
	        jac[h][edgeList[s]->black_index_in_x]=complex<double>(-1,0);
			jac[h][edgeList[s]->white_index_in_x]=complex<double>(1,0);  

	        ++h; 
			++t;
	      }


   



      //Below we take all regions, one by one, and make equations for each
	  for ( s=1; s<=(crossing_number+2); ++s )    //there are (crossing_number+2) regions in the link/knot
        {
        Quadrant *current_quadrant = baseQuadrantList[s];  //Each region is in one-to-one correspondence with baseQuadrant; current_cudrant is used for choosing the region; baseQuadrantList[i] has the crossing crossingList[i]
        Quadrant *region_quadrant = baseQuadrantList[s];    //used for taking all crossings in the chosen region	
 


		//Below make equations for a chosen region 
		if (current_quadrant->region_num_sides == 2)           //2-sided region, num_sides[i] = number of sides of region i    
		      {  fv[h] = current_quadrant->crossing->w - current_quadrant->next->crossing->w;  //crossing w - opposite crossing w = 0 
		 	     
		         //Compute partial derivatives
		         jac[h][current_quadrant->crossing->index_in_x]=complex<double>(1,0);
				 jac[h][current_quadrant->next->crossing->index_in_x]=complex<double>(-1,0);

		         ++h;
				 ++t;
				 
				 //Other equations in this region:
				 for (k=1; k<=current_quadrant->region_num_sides; ++k)
				      { if (region_quadrant->crossing->nw->region_index == region_quadrant->region_index)
				            { if ( region_quadrant->color == black )
				                 { 
								   fv[h] = region_quadrant->crossing->west->edge->u_black; //going clockwise in the region;edge numbers u_i*orientation of the edge
				                   jac[h][region_quadrant->crossing->west->edge->black_index_in_x]=complex<double>(1,0);
				                   ++h;
								   ++t;
				                 }
					          else
						         { 
								   fv[h] = region_quadrant->crossing->west->edge->u_white;  //see previous comment
				                   jac[h][region_quadrant->crossing->west->edge->white_index_in_x]=complex<double>(1,0);
                                   ++h; 
								   ++t;
				                 }
						     }
					    else if (region_quadrant->crossing->ne->region_index == region_quadrant->region_index)
				            { if ( region_quadrant->color == black )
				                 { 
								   fv[h] = region_quadrant->crossing->north->edge->u_black; //going clockwise in the region;edge numbers u_i*orientation of the edge
				                   for (p=0; p<n; ++p) 
				                         { if (p == region_quadrant->crossing->north->edge->black_index_in_x) jac[h][p]=complex<double>(1,0);}
				                   ++h;
								   ++t;
				                 }
					          else
						         { 
								   fv[h] = region_quadrant->crossing->north->edge->u_white;  //see previous comment
						           for (p=0; p<n; ++p) 
				                         { if (p == region_quadrant->crossing->north->edge->white_index_in_x) jac[h][p]=complex<double>(1,0);}
				                   ++h;
								   ++t;
				                 }
						     }
                        else if (region_quadrant->crossing->se->region_index == region_quadrant->region_index)
				            { if ( region_quadrant->color == black )
				                 { 
								   fv[h] = region_quadrant->crossing->east->edge->u_black; //going clockwise in the region;edge numbers u_i*orientation of the edge
				                   for (p=0; p<n; ++p) 
								      { if (p == region_quadrant->crossing->east->edge->black_index_in_x) jac[h][p]=complex<double>(1,0);}
						           ++h;
								   ++t;
						         }
					          else
						         { 
								   fv[h] = region_quadrant->crossing->east->edge->u_white;  //see previous comment
						           for (p=0; p<n; ++p) 
						               { if (p == region_quadrant->crossing->east->edge->white_index_in_x) jac[h][p]=complex<double>(1,0);}
								   ++h;
								   ++t;
						         }
						     }
                        else if (region_quadrant->crossing->sw->region_index == region_quadrant->region_index)
				            { if ( region_quadrant->color == black )
				                 {
								   fv[h] = region_quadrant->crossing->south->edge->u_black; //going clockwise in the region;edge numbers u_i*orientation of the edge
				                   for (p=0; p<n; ++p) 
						               { if (p == region_quadrant->crossing->south->edge->black_index_in_x) jac[h][p]=complex<double>(1,0);}
						           ++h;
								   ++t;
						         }
					          else
						         { 
								   fv[h] = region_quadrant->crossing->south->edge->u_white;  //see previous comment
						           for (p=0; p<n; ++p) 
						               { if (p == region_quadrant->crossing->south->edge->white_index_in_x) jac[h][p]=complex<double>(1,0);}
						           ++h;
								   ++t;
						         }
						     } 
				  
				        region_quadrant = region_quadrant->next;
				      }  //end of for
				 			 
		       } //end of if about 2-sided region
		

		else       //3 or more sided region
		       {  
                  help_massive_u = new  complex<double>[current_quadrant->region_num_sides+1];
                  help_massive_w = new  complex<double>[current_quadrant->region_num_sides+1];
                  //help_massive_ksi = new  complex<double>[current_quadrant->region_num_sides+1]; //shape parameters: may be will be used later
				  help_massive_u_index = new  int[current_quadrant->region_num_sides+1];  //u_index[i] is an index of u[i] in vector x
                  help_massive_w_index = new  int[current_quadrant->region_num_sides+1]; 
				  help_massive_eps = new  complex<double>[current_quadrant->region_num_sides+1]; //direction of the edge
			      
				  //Below put all u_i and w_i in this region in help_massive_u and help_massive_w
				  for (k=1; k<=current_quadrant->region_num_sides; ++k)
				      { help_massive_w[k] = region_quadrant->crossing->w;
                        help_massive_w_index[k] = region_quadrant->crossing->index_in_x;


                        //Partial Derivatives for 3-sided region
				        if (current_quadrant->region_num_sides == 3)
						    { jac[h+k-1][region_quadrant->crossing->index_in_x] = complex<double>(1,0);
						    }

                        region_quadrant = region_quadrant->next;  //going clockwise in the region
				      }  
                   
                  for (k=1; k<=current_quadrant->region_num_sides; ++k)
				      { if (region_quadrant->crossing->nw->region_index == region_quadrant->region_index)
				            { if ( region_quadrant->color == black )
				                 { help_massive_u[k] = region_quadrant->crossing->west->edge->u_black*complex<double>(region_quadrant->crossing->west->edge->edge_sense,0); //going clockwise in the region;edge numbers u_i*orientation of the edge
				                   help_massive_u_index[k] = region_quadrant->crossing->west->edge->black_index_in_x;
								   help_massive_eps[k] = complex<double>(region_quadrant->crossing->west->edge->edge_sense,0);
				                 }
					          else
						         { help_massive_u[k] = region_quadrant->crossing->west->edge->u_white*complex<double>(region_quadrant->crossing->west->edge->edge_sense,0);  //see previous comment
						           help_massive_u_index[k] = region_quadrant->crossing->west->edge->white_index_in_x;
                                   help_massive_eps[k] = complex<double>(region_quadrant->crossing->west->edge->edge_sense,0);
				                 }
						     }
					    else if (region_quadrant->crossing->ne->region_index == region_quadrant->region_index)
				            { if ( region_quadrant->color == black )
				                 { help_massive_u[k] = region_quadrant->crossing->north->edge->u_black*complex<double>(region_quadrant->crossing->north->edge->edge_sense,0); //going clockwise in the region;edge numbers u_i*orientation of the edge
				                   help_massive_u_index[k] = region_quadrant->crossing->north->edge->black_index_in_x;
                                   help_massive_eps[k] = complex<double>(region_quadrant->crossing->north->edge->edge_sense,0);
				                 }
					          else
						         { help_massive_u[k] = region_quadrant->crossing->north->edge->u_white*complex<double>(region_quadrant->crossing->north->edge->edge_sense,0);  //see previous comment
						           help_massive_u_index[k] = region_quadrant->crossing->north->edge->white_index_in_x;
                                   help_massive_eps[k] = complex<double>(region_quadrant->crossing->north->edge->edge_sense,0);
				                 }
						     }
                        else if (region_quadrant->crossing->se->region_index == region_quadrant->region_index)
				            { if ( region_quadrant->color == black )
				                 { help_massive_u[k] = region_quadrant->crossing->east->edge->u_black*complex<double>(region_quadrant->crossing->east->edge->edge_sense,0); //going clockwise in the region;edge numbers u_i*orientation of the edge
				                   help_massive_u_index[k] = region_quadrant->crossing->east->edge->black_index_in_x;
								   help_massive_eps[k] = complex<double>(region_quadrant->crossing->east->edge->edge_sense,0);
						         }
					          else
						         { help_massive_u[k] = region_quadrant->crossing->east->edge->u_white*complex<double>(region_quadrant->crossing->east->edge->edge_sense,0);  //see previous comment
						           help_massive_u_index[k] = region_quadrant->crossing->east->edge->white_index_in_x;
                                   help_massive_eps[k] = complex<double>(region_quadrant->crossing->east->edge->edge_sense,0);
						         }
						     }
                        else if (region_quadrant->crossing->sw->region_index == region_quadrant->region_index)
				            { if ( region_quadrant->color == black )
				                 { help_massive_u[k] = region_quadrant->crossing->south->edge->u_black*complex<double>(region_quadrant->crossing->south->edge->edge_sense,0); //going clockwise in the region;edge numbers u_i*orientation of the edge
				                   help_massive_u_index[k] = region_quadrant->crossing->south->edge->black_index_in_x;
                                   help_massive_eps[k] = complex<double>(region_quadrant->crossing->south->edge->edge_sense,0);
						         }
					          else
						         { help_massive_u[k] = region_quadrant->crossing->south->edge->u_white*complex<double>(region_quadrant->crossing->south->edge->edge_sense,0);  //see previous comment
						           help_massive_u_index[k] = region_quadrant->crossing->south->edge->white_index_in_x;
                                   help_massive_eps[k] = complex<double>(region_quadrant->crossing->south->edge->edge_sense,0);
						         }
						     }
				  
				        region_quadrant = region_quadrant->next;
				      }  
                  

				
				   //End of making equations

				  
//Compute residuals and partial derivatives for 3-sided region                       
				  if (current_quadrant->region_num_sides == 3)   //3-sided region
				      {
				        fv[h] = help_massive_w[1] - help_massive_u[3]*help_massive_u[1];
                        fv[h+1] = help_massive_w[2] - help_massive_u[1]*help_massive_u[2];
				        fv[h+2] = help_massive_w[3] - help_massive_u[2]*help_massive_u[3];
			  				

					    jac[h][help_massive_u_index[1]] = complex<double>(-1,0)*help_massive_u[3]*help_massive_eps[1];  //the derivatives with resp to u_i is -u_(i+1)*eps_i*eps_(i+1)
			            jac[h+1][help_massive_u_index[1]] = complex<double>(-1,0)*help_massive_u[2]*help_massive_eps[1];
						                
					    jac[h][help_massive_u_index[3]] = complex<double>(-1,0)*help_massive_u[1]*help_massive_eps[3]; //the derivatives with resp to u_(i+1) is -u_i*eps_i*eps_(i+1)
				        jac[h+2][help_massive_u_index[3]] = complex<double>(-1,0)*help_massive_u[2]*help_massive_eps[3];

					    jac[h+1][help_massive_u_index[2]] = complex<double>(-1,0)*help_massive_u[1]*help_massive_eps[2]; //the derivatives with resp to u_(i+1) is -u_i*eps_i*eps_(i+1)
						jac[h+2][help_massive_u_index[2]] = complex<double>(-1,0)*help_massive_u[3]*help_massive_eps[2];		            
						   	
				      }

                  else if (current_quadrant->region_num_sides == 4) //4-sided region
				     { fv[h] = help_massive_w[1]*help_massive_u[2]+help_massive_w[2]*help_massive_u[4]-help_massive_u[1]*help_massive_u[2]*help_massive_u[4];
					   fv[h+1] = help_massive_w[2]*help_massive_u[3]+help_massive_w[3]*help_massive_u[1]-help_massive_u[1]*help_massive_u[2]*help_massive_u[3];
					   fv[h+2] = help_massive_w[3]*help_massive_u[4]+help_massive_w[4]*help_massive_u[2]-help_massive_u[2]*help_massive_u[3]*help_massive_u[4];

					   //Derivatives of the first (i.e. h-th) equation
					   jac[h][help_massive_u_index[1]]= - help_massive_u[2]*help_massive_u[4]*help_massive_eps[1];
                       jac[h][help_massive_u_index[2]]= help_massive_w[1]*help_massive_eps[2]-help_massive_u[1]*help_massive_u[4]*help_massive_eps[2];
				       jac[h][help_massive_u_index[4]]= help_massive_w[2]*help_massive_eps[4]-help_massive_u[1]*help_massive_u[2]*help_massive_eps[4];
				       jac[h][help_massive_u_index[3]]= 0;
                       jac[h][help_massive_w_index[1]]= help_massive_u[2];
                       jac[h][help_massive_w_index[2]]= help_massive_u[4];
                       jac[h][help_massive_w_index[3]]= 0;
                       jac[h][help_massive_w_index[4]]= 0;

                       //Derivatives of the second (i.e. (h+1)-th) equation
					   jac[h+1][help_massive_u_index[2]]= - help_massive_u[3]*help_massive_u[1]*help_massive_eps[2];
                       jac[h+1][help_massive_u_index[1]]= help_massive_w[3]*help_massive_eps[1]-help_massive_u[2]*help_massive_u[3]*help_massive_eps[1];
				       jac[h+1][help_massive_u_index[3]]= help_massive_w[2]*help_massive_eps[3]-help_massive_u[1]*help_massive_u[2]*help_massive_eps[3];
				       jac[h+1][help_massive_u_index[4]]= 0;
                       jac[h+1][help_massive_w_index[3]]= help_massive_u[1];
                       jac[h+1][help_massive_w_index[2]]= help_massive_u[3];
                       jac[h+1][help_massive_w_index[1]]= 0;
                       jac[h+1][help_massive_w_index[4]]= 0;

                       //Derivatives of the third (i.e. (h+2)-th) equation
					   jac[h+2][help_massive_u_index[3]]= - help_massive_u[2]*help_massive_u[4]*help_massive_eps[3];
                       jac[h+2][help_massive_u_index[4]]= help_massive_w[3]*help_massive_eps[4]-help_massive_u[2]*help_massive_u[3]*help_massive_eps[4];
				       jac[h+2][help_massive_u_index[2]]= help_massive_w[4]*help_massive_eps[2]-help_massive_u[3]*help_massive_u[4]*help_massive_eps[2];
				       jac[h+2][help_massive_u_index[1]]= 0;
                       jac[h+2][help_massive_w_index[3]]= help_massive_u[4];
                       jac[h+2][help_massive_w_index[4]]= help_massive_u[2];
                       jac[h+2][help_massive_w_index[1]]= 0;
                       jac[h+2][help_massive_w_index[2]]= 0;

				     }

				  h=h+3;
				  t=t+3;

				  delete[] help_massive_u;
				  delete[] help_massive_w;
				  //delete[] help_massive_ksi;
				  delete[] help_massive_u_index;
                  delete[] help_massive_w_index;
				  delete[] help_massive_eps;
		    
			 }   //end of "else" about 3 or more sided regions
		       
          } //end of for (going region by region)     



  } //end of function f

