
/****************************************
Newton-Raphosn method. Contributor: Anastasiia Tsvietkobva, using ALGLIB
*****************************/






#include <cmath>
#include "link_diagram.h"
#include "cinverse.h"
#include "cqr.h"
#include <iostream>
using namespace std;



void Link::Newt(complex<double> *x,complex<double> *fv,int n,double eps,int *maxiter, complex<double> **jac) //Newton-Raphson method for overdetermined system (using QR decomposition)
{
    complex<double> tmp, delta, *x0, *Rx_leftside, *Rx_leftside_small;
    double dtmp, tmp1, tmp2;
    int i,j,k; 
    complex<double> **Qinv, **Rinv;   //rectangular jacobian from overdetermined system (+6 equations)
    ap::complex_2d_array cmatrix, jac_before_qr, Q, R, R_small;
    ap::complex_1d_array tau;

    x0 = new complex<double> [n];
    Rx_leftside = new complex<double>[n+6]; 
    Rx_leftside_small = new complex<double>[n]; 

    Qinv = new complex<double> *[n+6];   //for all (n+6) equations
	for (i=0;i<(n+6);i++) 
		      { Qinv[i] = new complex<double> [n+6];
              }	

    Rinv = new complex<double> *[n];   
	for (i=0;i<n;i++) 
		      { Rinv[i] = new complex<double> [n];
              }	

    for (k=0;k<*maxiter;k++)     //maxiter id the maximal number of iterations which you can change in main.cpp
	  { f(x,fv,n,jac);           // get residuals and jacobian for current value of x from ovedetermined system
    

        jac_before_qr.setlength(n+6,n);  //these are ALGLIB functions
	    Q.setlength(n+6,n+6);
	    R.setlength(n+6,n);
        R_small.setlength(n,n);
	
	    for(i = 0; i <= n+5; i++)
          {
           for(j = 0; j <= n-1; j++)
             {
			  jac_before_qr(i,j).x = real(jac[i][j]);
              jac_before_qr(i,j).y = imag(jac[i][j]);
             }
          }

	    int n2=n+6;
	    cmatrixqr(jac_before_qr, n2, n, tau);   //QR procedure from ALGLIB
        cmatrixqrunpackq(jac_before_qr, n2, n, tau, n2, Q);
        cmatrixqrunpackr(jac_before_qr, n2, n, R); 
    
//Find the inverse of Q
	    int NotSingQ = cmatrixinverse(Q, n2);   //inverse of Q, ALGLIB function
	    if (!NotSingQ) 
	       { cout<<"Matrix Q is singular"<<endl;
	         break;
	       }

//Store the inverse of Q in Qinv	
	    for(i = 0; i <= n+5; i++)   //return from ALGLIB types of variables to ours
            { for(j = 0; j <= n+5; j++)  
                   { tmp1 = Q(i,j).x;
					 tmp2 = Q(i,j).y;
 					 Qinv[i][j]=complex<double>(tmp1,tmp2);
                     //cout << jac[i][j];
                   }
              //cout <<endl <<endl;
             }

//Multiply Qinv by fv, get R*x=Qinv*help_fv
	   for(i = 0; i <= n+5; i++)
	      {  Rx_leftside[i]=complex<double>(0,0);
		     for (j = 0; j <= (n+5); j++)
                   {
                    Rx_leftside[i]=Rx_leftside[i]+Qinv[i][j]*fv[j];
                   }
                //if (k==1) cout<<x0[i]<<endl;
	       }
//cout<<"end of x0"<<endl;

//Rx_leftside delete last 6 rows, they are all 0
       for(i = 0; i <= n-1; i++)
           { tmp1 = real(Rx_leftside[i]);
             tmp2 = imag(Rx_leftside[i]);
			 Rx_leftside_small[i]=complex<double>(tmp1, tmp2);
            }

//R delete last 6 zero-rows
        for(i = 0; i <= n-1; i++)
            {  
   	           for (j = 0; j <= n-1; j++)
                   {
                    R_small(i,j)=R(i,j);
                   }
            }

//Find the inverse of R
	int NotSingR = cmatrixinverse(R, n);   //inverse of R
	if (!NotSingR) 
	   { cout<<"Matrix R is singular"<<endl;
	     break;
	   }

//Store the inverse of R in Rinv	
	for(i = 0; i <= n-1; i++)
            { for(j = 0; j <= n-1; j++)
                   { tmp1 = R(i,j).x;
					 tmp2 = R(i,j).y;
 					 Rinv[i][j]=complex<double>(tmp1,tmp2);
                     //cout << jac[i][j];
                   }
              //cout <<endl <<endl;
             }

//Multiply Rinv by Rx_leftside_small
	for(i = 0; i <= n-1; i++)
            {   x0[i]=complex<double>(0,0);
				for (j = 0; j <= n-1; j++)
                   {
                    x0[i]=x0[i]+Rinv[i][j]*Rx_leftside_small[j];
                   }
                //cout<<x0[i]<<endl;
	         }
		
// Update solution vector			
	dtmp = 0.0;
    for (i=0;i<n;i++) {
            dtmp += abs(x0[i]); 
            x[i] = x[i]-x0[i]; 
			//cout<<x[i]<<endl;	
		}

    if (dtmp < 0.000001) break;   //check if the solution is close enough: the precision 0.000001 here

    for (i=0;i<n;i++)
         { for (j=0;j<n;j++) 
	            {
                  jac[i][j] = complex<double>(0,0);  
				  fv[j]=complex<double>(0,0);
			    }
	     }	


} //end of big for
    *maxiter = k;  //that is how many iterations we did (may be less than maxiter, if the method converged fast)

    for (i=0;i<(n+6);i++) 
		     { delete [] Qinv[i];
             }
	delete [] Qinv;

    for (i=0;i<n;i++) 
		     { delete [] Rinv[i];
             }
	delete [] Rinv;

	delete [] x0;
    delete [] Rx_leftside;
    delete [] Rx_leftside_small;

}
