/*************************************************************************
Copyright (c) 1992-2007 The University of Tennessee.  All rights reserved.

Contributors:
    * Sergey Bochkanov (ALGLIB project). Translation from FORTRAN to
      pseudocode.

See subroutines comments for additional copyrights.

>>> SOURCE LICENSE >>>
This program is free software; you can redistribute it and/or modify
it under the terms of the GNU General Public License as published by
the Free Software Foundation (www.fsf.org); either version 2 of the 
License, or (at your option) any later version.

A copy of the GNU General Public License is available at
http://www.fsf.org/licensing/licenses

>>> END OF LICENSE >>>
*************************************************************************/

#ifndef _ctrinverse_h
#define _ctrinverse_h

#include "ap.h"
#include "ialglib.h"

/*************************************************************************
Complex triangular matrix inversion

The subroutine inverts the following types of matrices:
    * upper triangular
    * upper triangular with unit diagonal
    * lower triangular
    * lower triangular with unit diagonal

In case of an upper (lower) triangular matrix,  the  inverse  matrix  will
also be upper (lower) triangular, and after  the  end  of  the  algorithm,
the inverse matrix replaces the source matrix. The elements  below (above)
the main diagonal are not changed by the algorithm.

If the matrix has a unit diagonal, the inverse  matrix  also  has  a  unit
diagonal, the diagonal elements are not passed to the algorithm, they  are
not changed by the algorithm.

Input parameters:
    A       -   matrix.
                Array whose indexes range within [0..N-1,0..N-1].
    N       -   size of matrix A.
    IsUpper -   True, if the matrix is upper triangular.
    IsunitTriangular
            -   True, if the matrix has a unit diagonal.

Output parameters:
    A       -   inverse matrix (if the problem is not degenerate).

Result:
    True, if the matrix is not singular.
    False, if the matrix is singular.

  -- LAPACK routine (version 3.0) --
     Univ. of Tennessee, Univ. of California Berkeley, NAG Ltd.,
     Courant Institute, Argonne National Lab, and Rice University
     February 29, 1992
*************************************************************************/
bool cmatrixtrinverse(ap::complex_2d_array& a,
     int n,
     bool isupper,
     bool isunittriangular);


bool complexinvtriangular(ap::complex_2d_array& a,
     int n,
     bool isupper,
     bool isunittriangular);


#endif

