 
                /***************************************************************
A C++ program  for computing hyperbolic structure of hyperbolic knots and links with prime alternating diagrams where              every region has 2, 3 or 4 sides      
Anastasiia Tsvietkova, Morwen Thistlethwaite, ALGLIB library              ***************************************************************/


#include "link_diagram.h"
#include <cstdio>
#include <cstdlib>
#include <string>
#include <iostream>
using namespace std; 

FILE *fp, *gp;
int knotid;       //  (making this global is a cheap hack)  
complex<double> **jac;
double r;


int main( int argc, char **argv )  //Anastasiia: added return type "int" to main. Otherwise it is not ANSI C++
{
    int i2, t, p, i3, crossing_number, num_components;  
	
    fp = fopen( argv[1], "r" );
    gp = fopen( argv[2], "w" );

while ( fscanf( fp, "%d", &crossing_number) != EOF ) //read Dowker sequence from text file
    {
        std::vector<int> dt0;
        
        dt0.push_back( 0 );
       
        dt0[0] = crossing_number;
        
		fscanf( fp, "%d", &num_components );
		
		for ( i2=1; i2<crossing_number+num_components+2; ++i2 ) dt0.push_back( 0 );
		
		dt0[1] = num_components;
        
        for (i3=2; i3<num_components+2; ++i3)  
                    {
                      fscanf( fp, "%d", &t );   //t is component length
                      dt0[i3] = t;
                     }
    
        for ( i2=num_components+2; i2<crossing_number+num_components+2; ++i2 )
        {
            fscanf( fp, "%d", &t );
            dt0[i2] = t;
        }
        
		Link *link = new Link( dt0 );
    
        link->make_geometry();

        int n1 = 5*crossing_number + 6;  //n1 is the number of equations
        complex<double> *fv = new complex<double>[n1]; 
        complex<double> *x = new complex<double>[n1-6];
      
		link->initial_numbers(x);  //assigning initial values 
 
		int t = 10000;  //maximal number of iterations; feel free to change it

        jac = new complex<double> *[n1];
		for (p=0;p<n1;p++) 
		      { jac[p] = new complex<double> [n1-6];
              }		
		
		link->Newt(x,fv,(n1-6),0.001e-0010,&t, jac);  //do Newton's method, get the solution x: crossing #s, black edge #s and white edge #s 

        delete[] fv;
        
		for (p=0;p<n1;p++) 
		     { delete [] jac[p];
             }
		delete[] jac; 
		
        delete link;

		fprintf(gp,"Crossing numbers:\n");
        for ( p=0; p<crossing_number; ++p )
            {
				fprintf( gp, "%9f%+9f \n", real(x[p]), imag(x[p]) );
		    }
		fprintf(gp,"Edge numbers in the black regions:\n");
        for ( p=crossing_number; p<3*crossing_number; ++p )
            {
				fprintf( gp, "%9f%+9f \n", real(x[p]), imag(x[p]) );
		    }
		fprintf(gp,"Edge numbers in the white regions:\n");
        for ( p=3*crossing_number; p<5*crossing_number; ++p )
            {
				fprintf( gp, "%9f%+9f \n", real(x[p]), imag(x[p]) );
		    }

        
		delete[] x;

    }
    
}